@@ -5,33 +5,32 @@
 
     When it comes to combining multiple controller or view functions (however
     you want to call them) you need a dispatcher.  A simple way would be
-    applying regular expression tests on the ``PATH_INFO`` and call registered
-    callback functions that return the value then.
+    applying regular expression tests on the ``PATH_INFO`` and calling
+    registered callback functions that return the value then.
 
     This module implements a much more powerful system than simple regular
     expression matching because it can also convert values in the URLs and
     build URLs.
 
     Here a simple example that creates an URL map for an application with
-    two subdomains (www and kb) and some URL rules::
+    two subdomains (www and kb) and some URL rules:
 
-        m = Map([
-            # Static URLs
-            Rule('/', endpoint='static/index'),
-            Rule('/about', endpoint='static/about'),
-            Rule('/help', endpoint='static/help'),
-
-            # Knowledge Base
-            Subdomain('kb', [
-                Rule('/', endpoint='kb/index'),
-                Rule('/browse/', endpoint='kb/browse'),
-                Rule('/browse/<int:id>/', endpoint='kb/browse'),
-                Rule('/browse/<int:id>/<int:page>', endpoint='kb/browse')
-            ])
-        ], default_subdomain='www')
+    >>> m = Map([
+    ...     # Static URLs
+    ...     Rule('/', endpoint='static/index'),
+    ...     Rule('/about', endpoint='static/about'),
+    ...     Rule('/help', endpoint='static/help'),
+    ...     # Knowledge Base
+    ...     Subdomain('kb', [
+    ...         Rule('/', endpoint='kb/index'),
+    ...         Rule('/browse/', endpoint='kb/browse'),
+    ...         Rule('/browse/<int:id>/', endpoint='kb/browse'),
+    ...         Rule('/browse/<int:id>/<int:page>', endpoint='kb/browse')
+    ...     ])
+    ... ], default_subdomain='www')
 
     If the application doesn't use subdomains it's perfectly fine to not set
-    the default subdomain and use the `Subdomain` rule factory.  The endpoint
+    the default subdomain and not use the `Subdomain` rule factory.  The endpoint
     in the rules can be anything, for example import paths or unique
     identifiers.  The WSGI application can use those endpoints to get the
     handler for that URL.  It doesn't have to be a string at all but it's
@@ -48,12 +47,14 @@
     >>> c.build("kb/browse", dict(id=42, page=3))
     'http://kb.example.com/browse/42/3'
     >>> c.build("static/about")
-    u'/about'
-    >>> c.build("static/about", subdomain="kb")
-    'http://www.example.com/about'
+    '/about'
     >>> c.build("static/index", force_external=True)
     'http://www.example.com/'
 
+    >>> c = m.bind('example.com', subdomain='kb')
+    >>> c.build("static/about")
+    'http://www.example.com/about'
+
     The first argument to bind is the server name *without* the subdomain.
     Per default it will assume that the script is mounted on the root, but
     often that's not the case so you can provide the real mount point as
@@ -91,21 +92,19 @@
     method is raised.
 
 
-    :copyright: 2007-2008 by Armin Ronacher, Leif K-Brooks,
+    :copyright: (c) 2010 by the Werkzeug Team, see AUTHORS for more details.
                              Thomas Johansson.
     :license: BSD, see LICENSE for more details.
 """
-import sys
 import re
+from pprint import pformat
 from urlparse import urljoin
 from itertools import izip
 
-from werkzeug.utils import url_encode, url_quote, redirect, format_string
+from werkzeug.urls import url_encode, url_quote
+from werkzeug.utils import redirect, format_string
 from werkzeug.exceptions import HTTPException, NotFound, MethodNotAllowed
-try:
-    set
-except NameError:
-    from sets import Set as set
+from werkzeug._internal import _get_environ
 
 
 _rule_re = re.compile(r'''
@@ -172,8 +171,8 @@ def get_converter(map, name, args):
 
 
 class RoutingException(Exception):
-    """Special exceptions that require the application to redirect, notifies
-    him about missing urls etc.
+    """Special exceptions that require the application to redirect, notifying
+    about missing urls, etc.
 
     :internal:
     """
@@ -181,9 +180,9 @@ class RoutingException(Exception):
 
 class RequestRedirect(HTTPException, RoutingException):
     """Raise if the map requests a redirect. This is for example the case if
-    `strict_slashes` are activated and an url that requires a leading slash.
+    `strict_slashes` are activated and an url that requires a trailing slash.
 
-    The attribute `new_url` contains the absolute desitination url.
+    The attribute `new_url` contains the absolute destination url.
     """
     code = 301
 
@@ -243,8 +242,8 @@ class Subdomain(RuleFactory):
             ])
         ])
 
-    All the rules except of the ``'#select_language'`` endpoint will now
-    listen on a two letter long subdomain that helds the language code
+    All the rules except for the ``'#select_language'`` endpoint will now
+    listen on a two letter long subdomain that holds the language code
     for the current request.
     """
 
@@ -255,6 +254,7 @@ class Subdomain(RuleFactory):
     def get_rules(self, map):
         for rulefactory in self.rules:
             for rule in rulefactory.get_rules(map):
+                rule = rule.empty()
                 rule.subdomain = self.subdomain
                 yield rule
 
@@ -280,6 +280,7 @@ class Submount(RuleFactory):
     def get_rules(self, map):
         for rulefactory in self.rules:
             for rule in rulefactory.get_rules(map):
+                rule = rule.empty()
                 rule.rule = self.path + rule.rule
                 yield rule
 
@@ -304,6 +305,7 @@ class EndpointPrefix(RuleFactory):
     def get_rules(self, map):
         for rulefactory in self.rules:
             for rule in rulefactory.get_rules(map):
+                rule = rule.empty()
                 rule.endpoint = self.prefix + rule.endpoint
                 yield rule
 
@@ -374,7 +376,7 @@ class RuleTemplateFactory(RuleFactory):
 class Rule(RuleFactory):
     """A Rule represents one URL pattern.  There are some options for `Rule`
     that change the way it behaves and are passed to the `Rule` constructor.
-    Note that beside the rule-string all arguments *must* be keyword arguments
+    Note that besides the rule-string all arguments *must* be keyword arguments
     in order to not break the application on Werkzeug upgrades.
 
     `string`
@@ -393,7 +395,7 @@ class Rule(RuleFactory):
     `endpoint`
         The endpoint for this rule. This can be anything. A reference to a
         function, a string, a number etc.  The preferred way is using a string
-        as because the endpoint is used for URL generation.
+        because the endpoint is used for URL generation.
 
     `defaults`
         An optional dict with defaults for other rules with the same endpoint.
@@ -428,14 +430,22 @@ class Rule(RuleFactory):
         endpoints for `POST` and `GET`.  If methods are defined and the path
         matches but the method matched against is not in this list or in the
         list of another rule for that path the error raised is of the type
-        `MethodNotAllowed` rather than `NotFound`.
+        `MethodNotAllowed` rather than `NotFound`.  If `GET` is present in the
+        list of methods and `HEAD` is not, `HEAD` is added automatically.
+
+        .. versionchanged:: 0.6.1
+           `HEAD` is now automatically added to the methods if `GET` is
+           present.  The reason for this is that existing code often did not
+           work properly in servers not rewriting `HEAD` to `GET`
+           automatically and it was not documented how `HEAD` should be
+           treated.  This was considered a bug in Werkzeug because of that.
 
     `strict_slashes`
         Override the `Map` setting for `strict_slashes` only for this rule. If
         not specified the `Map` setting is used.
 
     `build_only`
-        Set this to true and the rule will never match but will create a URL
+        Set this to True and the rule will never match but will create a URL
         that can be build. This is useful if you have resources on a subdomain
         or folder that are not handled by the WSGI application (like static data)
 
@@ -482,36 +492,46 @@ class Rule(RuleFactory):
             self.methods = None
         else:
             self.methods = set([x.upper() for x in methods])
+            if 'HEAD' not in self.methods and 'GET' in self.methods:
+                self.methods.add('HEAD')
         self.endpoint = endpoint
         self.greediness = 0
         self.redirect_to = redirect_to
 
-        self._trace = []
         if defaults is not None:
             self.arguments = set(map(str, defaults))
         else:
             self.arguments = set()
-        self._converters = {}
-        self._regex = None
-        self._weights = []
+        self._trace = self._converters = self._regex = self._weights = None
 
     def empty(self):
         """Return an unbound copy of this rule.  This can be useful if you
         want to reuse an already bound URL for another map."""
-        return Rule(self.rule, self.defaults, self.subdomain, self.methods,
+        defaults = None
+        if self.defaults is not None:
+            defaults = dict(self.defaults)
+        return Rule(self.rule, defaults, self.subdomain, self.methods,
                     self.build_only, self.endpoint, self.strict_slashes,
                     self.redirect_to)
 
     def get_rules(self, map):
         yield self
 
-    def bind(self, map):
+    def refresh(self):
+        """Rebinds and refreshes the URL.  Call this if you modified the
+        rule in place.
+
+        :internal:
+        """
+        self.bind(self.map, rebind=True)
+
+    def bind(self, map, rebind=False):
         """Bind the url to a map and create a regular expression based on
         the information from the rule itself and the defaults from the map.
 
         :internal:
         """
-        if self.map is not None:
+        if self.map is not None and not rebind:
             raise RuntimeError('url rule %r already bound to map %r' %
                                (self, self.map))
         self.map = map
@@ -523,6 +543,10 @@ class Rule(RuleFactory):
         rule = self.subdomain + '|' + (self.is_leaf and self.rule
                                        or self.rule.rstrip('/'))
 
+        self._trace = []
+        self._converters = {}
+        self._weights = []
+
         regex_parts = []
         for converter, arguments, variable in parse_rule(rule):
             if converter is None:
@@ -585,7 +609,7 @@ class Rule(RuleFactory):
                     result.update(self.defaults)
                 return result
 
-    def build(self, values):
+    def build(self, values, append_unknown=True):
         """Assembles the relative url for that rule and the subdomain.
         If building doesn't work for some reasons `None` is returned.
 
@@ -605,11 +629,16 @@ class Rule(RuleFactory):
                 add(data)
         subdomain, url = (u''.join(tmp)).split('|', 1)
 
-        query_vars = {}
-        for key in set(values) - processed:
-            query_vars[key] = unicode(values[key])
-        if query_vars:
-            url += '?' + url_encode(query_vars, self.map.charset)
+        if append_unknown:
+            query_vars = MultiDict(values)
+            for key in processed:
+                if key in query_vars:
+                    del query_vars[key]
+
+            if query_vars:
+                url += '?' + url_encode(query_vars, self.map.charset,
+                                        sort=self.map.sort_parameters,
+                                        key=self.map.sort_key)
 
         return subdomain, url
 
@@ -622,13 +651,14 @@ class Rule(RuleFactory):
                self.endpoint == rule.endpoint and self != rule and \
                self.arguments == rule.arguments
 
-    def suitable_for(self, values, method):
+    def suitable_for(self, values, method=None):
         """Check if the dict of values has enough data for url generation.
 
         :internal:
         """
-        if self.methods is not None and method not in self.methods:
-            return False
+        if method is not None:
+            if self.methods is not None and method not in self.methods:
+                return False
 
         valueset = set(values)
 
@@ -755,14 +785,20 @@ class BaseConverter(object):
 
 class UnicodeConverter(BaseConverter):
     """This converter is the default converter and accepts any string but
-    only one one path segment.  Thus the string can not include a slash.
+    only one path segment.  Thus the string can not include a slash.
 
-    Supported arguments:
+    This is the default validator.
 
-    - `minlength` - the minimum length of the string. must be greater
-      than 1.
-    - `maxlength` - the maximum length of the string.
-    - `length` - the exact length of that string.
+    Example::
+
+        Rule('/pages/<page>'),
+        Rule('/<string(length=2):lang_code>')
+
+    :param map: the :class:`Map`.
+    :param minlength: the minimum length of the string.  Must be greater
+                      or equal 1.
+    :param maxlength: the maximum length of the string.
+    :param length: the exact length of the string.
     """
 
     def __init__(self, map, minlength=1, maxlength=None, length=None):
@@ -786,6 +822,10 @@ class AnyConverter(BaseConverter):
     identifiers or unicode strings::
 
         Rule('/<any(about, help, imprint, u"class"):page_name>')
+
+    :param map: the :class:`Map`.
+    :param items: this function accepts the possible items as positional
+                  arguments.
     """
 
     def __init__(self, map, *items):
@@ -794,7 +834,14 @@ class AnyConverter(BaseConverter):
 
 
 class PathConverter(BaseConverter):
-    """Like the default string converter, but it also matches slashes."""
+    """Like the default :class:`UnicodeConverter`, but it also matches
+    slashes.  This is useful for wikis and similar applications::
+
+        Rule('/<path:wikipage>')
+        Rule('/<path:wikipage>/edit')
+
+    :param map: the :class:`Map`.
+    """
     regex = '[^/].*?'
     is_greedy = True
     weight = 50
@@ -833,14 +880,15 @@ class IntegerConverter(NumberConverter):
 
         Rule('/page/<int:page>')
 
-    Supported arguments:
+    This converter does not support negative values.
 
-    - `fixed_digits` - the number of fixed digits in the URL. If you
-      set this to ``4`` for example, the application will only match
-      if the url looks like ``/0001/``.  The default is
-      variable length.
-    - `min` - the minimal value.
-    - `max` - the maximal value.
+    :param map: the :class:`Map`.
+    :param fixed_digits: the number of fixed digits in the URL.  If you set
+                         this to ``4`` for example, the application will
+                         only match if the url looks like ``/0001/``.  The
+                         default is variable length.
+    :param min: the minimal value.
+    :param max: the maximal value.
     """
     regex = r'\d+'
     num_convert = int
@@ -851,10 +899,11 @@ class FloatConverter(NumberConverter):
 
         Rule('/probability/<float:probability>')
 
-    Supported arguments:
+    This converter does not support negative values.
 
-    - `min` - the minimal value.
-    - `max` - the maximal value.
+    :param map: the :class:`Map`.
+    :param min: the minimal value.
+    :param max: the maximal value.
     """
     regex = r'\d+\.\d+'
     num_convert = float
@@ -868,26 +917,34 @@ class Map(object):
     parameters.  Some of the configuration values are only stored on the
     `Map` instance since those affect all rules, others are just defaults
     and can be overridden for each rule.  Note that you have to specify all
-    arguments beside the `rules` as keywords arguments!
+    arguments besides the `rules` as keyword arguments!
+
+    :param rules: sequence of url rules for this map.
+    :param default_subdomain: The default subdomain for rules without a
+                              subdomain defined.
+    :param charset: charset of the url. defaults to ``"utf-8"``
+    :param strict_slashes: Take care of trailing slashes.
+    :param redirect_defaults: This will redirect to the default rule if it
+                              wasn't visited that way. This helps creating
+                              unique URLs.
+    :param converters: A dict of converters that adds additional converters
+                       to the list of converters. If you redefine one
+                       converter this will override the original one.
+    :param sort_parameters: If set to `True` the url parameters are sorted.
+                            See `url_encode` for more details.
+    :param sort_key: The sort key function for `url_encode`.
+
+    .. versionadded:: 0.5
+        `sort_parameters` and `sort_key` was added.
     """
 
+    #: .. versionadded:: 0.6
+    #:    a dict of default converters to be used.
+    default_converters = None
+
     def __init__(self, rules=None, default_subdomain='', charset='utf-8',
                  strict_slashes=True, redirect_defaults=True,
-                 converters=None):
-        """Initializes the new URL map.
-
-        :param rules: sequence of url rules for this map.
-        :param default_subdomain: The default subdomain for rules without a
-                                  subdomain defined.
-        :param charset: charset of the url. defaults to ``"utf-8"``
-        :param strict_slashes: Take care of trailing slashes.
-        :param redirect_defaults: This will redirect to the default rule if it
-                                  wasn't visited that way. This helps creating
-                                  unique URLs.
-        :param converters: A dict of converters that adds additional converters
-                           to the list of converters. If you redefine one
-                           converter this will override the original one.
-        """
+                 converters=None, sort_parameters=False, sort_key=None):
         self._rules = []
         self._rules_by_endpoint = {}
         self._remap = True
@@ -897,10 +954,13 @@ class Map(object):
         self.strict_slashes = strict_slashes
         self.redirect_defaults = redirect_defaults
 
-        self.converters = DEFAULT_CONVERTERS.copy()
+        self.converters = self.default_converters.copy()
         if converters:
             self.converters.update(converters)
 
+        self.sort_parameters = sort_parameters
+        self.sort_key = sort_key
+
         for rulefactory in rules or ():
             self.add(rulefactory)
 
@@ -911,6 +971,11 @@ class Map(object):
         you want to wrap the builder a bit so that the current language
         code is automatically added if not provided but endpoints expect
         it.
+
+        :param endpoint: the endpoint to check.
+        :param arguments: this function accepts one or more arguments
+                          as positional arguments.  Each one of them is
+                          checked.
         """
         self.update()
         arguments = set(arguments)
@@ -920,7 +985,12 @@ class Map(object):
         return False
 
     def iter_rules(self, endpoint=None):
-        """Iterate over all rules or the rules of an endpoint."""
+        """Iterate over all rules or the rules of an endpoint.
+
+        :param endpoint: if provided only the rules for that endpoint
+                         are returned.
+        :return: an iterator
+        """
         if endpoint is not None:
             return iter(self._rules_by_endpoint[endpoint])
         return iter(self._rules)
@@ -928,6 +998,8 @@ class Map(object):
     def add(self, rulefactory):
         """Add a new rule or factory to the map and bind it.  Requires that the
         rule is not bound to another map.
+
+        :param rulefactory: a :class:`Rule` or :class:`RuleFactory`
         """
         for rule in rulefactory.get_rules(self):
             rule.bind(self)
@@ -935,21 +1007,16 @@ class Map(object):
             self._rules_by_endpoint.setdefault(rule.endpoint, []).append(rule)
         self._remap = True
 
-    def add_rule(self, rule):
-        from warnings import warn
-        warn(DeprecationWarning('use map.add instead of map.add_rule now'))
-        return self.add(rule)
-
     def bind(self, server_name, script_name=None, subdomain=None,
              url_scheme='http', default_method='GET', path_info=None):
-        """Return a new `MapAdapter` with the details specified to the call.
-        Note that `script_name` will default to ``'/'`` if not further
+        """Return a new :class:`MapAdapter` with the details specified to the
+        call.  Note that `script_name` will default to ``'/'`` if not further
         specified or `None`.  The `server_name` at least is a requirement
         because the HTTP RFC requires absolute URLs for redirects and so all
         redirect exceptions raised by Werkzeug will contain the full canonical
         URL.
 
-        If no path_info is passed to match() it will use the default path
+        If no path_info is passed to :meth:`match` it will use the default path
         info passed to bind.  While this doesn't really make sense for
         manual bind calls, it's useful if you bind a map to a WSGI
         environment which already contains the path info.
@@ -965,10 +1032,9 @@ class Map(object):
         return MapAdapter(self, server_name, script_name, subdomain,
                           url_scheme, path_info, default_method)
 
-    def bind_to_environ(self, environ, server_name=None, subdomain=None,
-                        calculate_subdomain=False):
-        """Like `bind` but you can pass it an WSGI environment and it will
-        fetch the information from that directory.  Note that because of
+    def bind_to_environ(self, environ, server_name=None, subdomain=None):
+        """Like :meth:`bind` but you can pass it an WSGI environment and it
+        will fetch the information from that dictionary.  Note that because of
         limitations in the protocol there is no way to get the current
         subdomain and real `server_name` from the environment.  If you don't
         provide it, Werkzeug will use `SERVER_NAME` and `SERVER_PORT` (or
@@ -981,14 +1047,22 @@ class Map(object):
         in the wsgi `environ` is ``'staging.dev.example.com'`` the calculated
         subdomain will be ``'staging.dev'``.
 
-        If the object passed as environ as an environ attribute, the value of
+        If the object passed as environ has an environ attribute, the value of
         this attribute is used instead.  This allows you to pass request
-        objects.  Additionally `PATH_INFO` added as a default ot the
-        `MapAdapter` so that you don't have to pass the path info to the
-        match method.
+        objects.  Additionally `PATH_INFO` added as a default of the
+        :class:`MapAdapter` so that you don't have to pass the path info to
+        the match method.
+
+        .. versionchanged:: 0.5
+            previously this method accepted a bogus `calculate_subdomain`
+            parameter that did not have any effect.  It was removed because
+            of that.
+
+        :param environ: a WSGI environment.
+        :param server_name: an optional server name hint (see above).
+        :param subdomain: optionally the current subdomain (see above).
         """
-        if hasattr(environ, 'environ'):
-            environ = environ.environ
+        environ = _get_environ(environ)
         if server_name is None:
             if 'HTTP_HOST' in environ:
                 server_name = environ['HTTP_HOST']
@@ -998,14 +1072,15 @@ class Map(object):
                    in (('https', '443'), ('http', '80')):
                     server_name += ':' + environ['SERVER_PORT']
         elif subdomain is None:
-            cur_server_name = environ['SERVER_NAME'].split('.')
+            wsgi_server_name = environ.get('HTTP_HOST', environ['SERVER_NAME'])
+            cur_server_name = wsgi_server_name.split(':', 1)[0].split('.')
             real_server_name = server_name.split(':', 1)[0].split('.')
             offset = -len(real_server_name)
             if cur_server_name[offset:] != real_server_name:
                 raise ValueError('the server name provided (%r) does not '
                                  'match the server name from the WSGI '
                                  'environment (%r)' %
-                                 (environ['SERVER_NAME'], server_name))
+                                 (server_name, wsgi_server_name))
             subdomain = '.'.join(filter(None, cur_server_name[:offset]))
         return Map.bind(self, server_name, environ.get('SCRIPT_NAME'),
                         subdomain, environ['wsgi.url_scheme'],
@@ -1022,9 +1097,15 @@ class Map(object):
             self._remap = False
 
 
+    def __repr__(self):
+        rules = self.iter_rules()
+        return '%s([%s])' % (self.__class__.__name__, pformat(list(rules)))
+
+
+
 class MapAdapter(object):
-    """Retured by `Map.bind` or `Map.bind_to_environ` and does the
-    URL matching and building based on runtime information.
+    """Returned by :meth:`Map.bind` or :meth:`Map.bind_to_environ` and does
+    the URL matching and building based on runtime information.
     """
 
     def __init__(self, map, server_name, script_name, subdomain,
@@ -1044,7 +1125,7 @@ class MapAdapter(object):
         """Does the complete dispatching process.  `view_func` is called with
         the endpoint and a dict with the values for the view.  It should
         look up the view function, call it, and return a response object
-        or WSGI application.  http exceptions are not catched by default
+        or WSGI application.  http exceptions are not caught by default
         so that applications can display nicer error messages by just
         catching them by hand.  If you want to stick with the default
         error messages you can pass it ``catch_http_exceptions=True`` and
@@ -1068,8 +1149,19 @@ class MapAdapter(object):
                 return urls.dispatch(lambda e, v: views[e](request, **v),
                                      catch_http_exceptions=True)
 
-        Keep in mind that this method might return exception objects too, so
-        use `Response.force_type` to get a response object.
+        Keep in mind that this method might return exception objects, too, so
+        use :class:`Response.force_type` to get a response object.
+
+        :param view_func: a function that is called with the endpoint as
+                          first argument and the value dict as second.  Has
+                          to dispatch to the actual view function with this
+                          information.  (see above)
+        :param path_info: the path info to use for matching.  Overrides the
+                          path info specified on binding.
+        :param method: the HTTP method used for matching.  Overrides the
+                       method specified on binding.
+        :param catch_http_exceptions: set to `True` to catch any of the
+                                      werkzeug :class:`HTTPException`\s.
         """
         try:
             try:
@@ -1082,7 +1174,7 @@ class MapAdapter(object):
                 return e
             raise
 
-    def match(self, path_info=None, method=None):
+    def match(self, path_info=None, method=None, return_rule=False):
         """The usage is simple: you just pass the match method the current
         path info as well as the method (which defaults to `GET`).  The
         following things can then happen:
@@ -1093,18 +1185,19 @@ class MapAdapter(object):
           same object as `werkzeug.exceptions.NotFound`)
 
         - you receive a `MethodNotAllowed` exception that indicates that there
-          is a match for this URL but non for the current request method.
+          is a match for this URL but not for the current request method.
           This is useful for RESTful applications.
 
         - you receive a `RequestRedirect` exception with a `new_url`
           attribute.  This exception is used to notify you about a request
-          Werkzeug requests by your WSGI application.  This is for example the
+          Werkzeug requests from your WSGI application.  This is for example the
           case if you request ``/foo`` although the correct URL is ``/foo/``
           You can use the `RequestRedirect` instance as response-like object
           similar to all other subclasses of `HTTPException`.
 
-        - you get a tuple in the form ``(endpoint, arguments)`` when there is
-          a match.
+        - you get a tuple in the form ``(endpoint, arguments)`` if there is
+          a match (unless `return_rule` is True, in which case you get a tuple
+          in the form ``(rule, arguments)``)
 
         If the path info is not passed to the match method the default path
         info of the map is used (defaults to the root URL if not defined
@@ -1116,10 +1209,9 @@ class MapAdapter(object):
 
         Here is a small example for matching:
 
-        >>> from werkzeug.routing import Map, Rule
         >>> m = Map([
         ...     Rule('/', endpoint='index'),
-        ...     Rule('/downloads/', endpoint='downloads/index'), 
+        ...     Rule('/downloads/', endpoint='downloads/index'),
         ...     Rule('/downloads/<int:id>', endpoint='downloads/show')
         ... ])
         >>> urls = m.bind("example.com", "/")
@@ -1133,11 +1225,21 @@ class MapAdapter(object):
         >>> urls.match("/downloads")
         Traceback (most recent call last):
           ...
-        werkzeug.routing.RequestRedirect: http://example.com/downloads/
+        RequestRedirect: http://example.com/downloads/
         >>> urls.match("/missing")
         Traceback (most recent call last):
           ...
-        werkzeug.routing.NotFound: /missing
+        NotFound: 404 Not Found
+
+        :param path_info: the path info to use for matching.  Overrides the
+                          path info specified on binding.
+        :param method: the HTTP method used for matching.  Overrides the
+                       method specified on binding.
+        :param return_rule: return the rule that matched instead of just the
+                            endpoint (defaults to `False`).
+
+        .. versionadded:: 0.6
+            `return_rule` was added.
         """
         self.map.update()
         if path_info is None:
@@ -1156,7 +1258,7 @@ class MapAdapter(object):
                     self.subdomain and self.subdomain + '.' or '',
                     self.server_name,
                     self.script_name[:-1],
-                    path_info.lstrip('/')
+                    url_quote(path_info.lstrip('/'), self.map.charset)
                 )))
             if rv is None:
                 continue
@@ -1174,7 +1276,7 @@ class MapAdapter(object):
                             subdomain and subdomain + '.' or '',
                             self.server_name,
                             self.script_name[:-1],
-                            path.lstrip('/')
+                            url_quote(path.lstrip('/'), self.map.charset)
                         )))
             if rule.redirect_to is not None:
                 if isinstance(rule.redirect_to, basestring):
@@ -1191,7 +1293,10 @@ class MapAdapter(object):
                     self.server_name,
                     self.script_name
                 ), redirect_url)))
-            return rule.endpoint, rv
+            if return_rule:
+                return rule, rv
+            else:
+                return rule.endpoint, rv
         if have_match_for:
             raise MethodNotAllowed(valid_methods=list(have_match_for))
         raise NotFound()
@@ -1199,6 +1304,11 @@ class MapAdapter(object):
     def test(self, path_info=None, method=None):
         """Test if a rule would match.  Works like `match` but returns `True`
         if the URL matches, or `False` if it does not exist.
+
+        :param path_info: the path info to use for matching.  Overrides the
+                          path info specified on binding.
+        :param method: the HTTP method used for matching.  Overrides the
+                       method specified on binding.
         """
         try:
             self.match(path_info, method)
@@ -1208,7 +1318,29 @@ class MapAdapter(object):
             return False
         return True
 
-    def build(self, endpoint, values=None, method=None, force_external=False):
+    def _partial_build(self, endpoint, values, method, append_unknown):
+        """Helper for :meth:`build`.  Returns subdomain and path for the
+        rule that accepts this endpoint, values and method.
+
+        :internal:
+        """
+        # in case the method is none, try with the default method first
+        if method is None:
+            rv = self._partial_build(endpoint, values, self.default_method,
+                                     append_unknown)
+            if rv is not None:
+                return rv
+
+        # default method did not match or a specific method is passed,
+        # check all and go with first result.
+        for rule in self.map._rules_by_endpoint.get(endpoint, ()):
+            if rule.suitable_for(values, method):
+                rv = rule.build(values, append_unknown)
+                if rv is not None:
+                    return rv
+
+    def build(self, endpoint, values=None, method=None, force_external=False,
+              append_unknown=True):
         """Building URLs works pretty much the other way round.  Instead of
         `match` you call `build` and pass it the endpoint and a dict of
         arguments for the placeholders.
@@ -1216,12 +1348,14 @@ class MapAdapter(object):
         The `build` function also accepts an argument called `force_external`
         which, if you set it to `True` will force external URLs. Per default
         external URLs (include the server name) will only be used if the
-        target URL is on a
-        different subdomain.
-
-        With the same map as in the example above this code generates some
-        target URLs:
+        target URL is on a different subdomain.
 
+        >>> m = Map([
+        ...     Rule('/', endpoint='index'),
+        ...     Rule('/downloads/', endpoint='downloads/index'),
+        ...     Rule('/downloads/<int:id>', endpoint='downloads/show')
+        ... ])
+        >>> urls = m.bind("example.com", "/")
         >>> urls.build("index", {})
         '/'
         >>> urls.build("downloads/show", {'id': 42})
@@ -1243,24 +1377,38 @@ class MapAdapter(object):
         raised.
 
         The build method accepts an argument called `method` which allows you
-        to specify the method you want to have an URL builded for if you have
+        to specify the method you want to have an URL built for if you have
         different methods for the same endpoint specified.
+
+        .. versionadded:: 0.6
+           the `append_unknown` parameter was added.
+
+        :param endpoint: the endpoint of the URL to build.
+        :param values: the values for the URL to build.  Unhandled values are
+                       appended to the URL as query parameters.
+        :param method: the HTTP method for the rule if there are different
+                       URLs for different methods on the same endpoint.
+        :param force_external: enforce full canonical external URLs.
+        :param append_unknown: unknown parameters are appended to the generated
+                               URL as query string argument.  Disable this
+                               if you want the builder to ignore those.
         """
         self.map.update()
-        method = method or self.default_method
         if values:
-            values = dict([(k, v) for k, v in values.items() if v is not None])
+            if isinstance(values, MultiDict):
+                values = dict((k, v) for k, v in values.iteritems(multi=True)
+                              if v is not None)
+            else:
+                values = dict((k, v) for k, v in values.iteritems()
+                              if v is not None)
         else:
             values = {}
 
-        for rule in self.map._rules_by_endpoint.get(endpoint, ()):
-            if rule.suitable_for(values, method):
-                rv = rule.build(values)
-                if rv is not None:
-                    break
-        else:
+        rv = self._partial_build(endpoint, values, method, append_unknown)
+        if rv is None:
             raise BuildError(endpoint, values, method)
         subdomain, path = rv
+
         if not force_external and subdomain == self.subdomain:
             return str(urljoin(self.script_name, path.lstrip('/')))
         return str('%s://%s%s%s/%s' % (
@@ -1281,3 +1429,6 @@ DEFAULT_CONVERTERS = {
     'int':              IntegerConverter,
     'float':            FloatConverter
 }
+
+from werkzeug.datastructures import ImmutableDict, MultiDict
+Map.default_converters = ImmutableDict(DEFAULT_CONVERTERS)
